import logging
import os
import sys

from pysmt.shortcuts import Symbol, And, Not, Or

from mc308.logics.bool import BoolFormula
from mc308.auto.ts import TS, FinitePath
from mc308.parsers.ts_parser import TSParser
from mc308.auto.visualization import ts_to_dot, to_png

from mc308.td1 import check_invar, async_product


def test_ex1():
    print("Running test_ex1...")
    p = TSParser()
    p1,p2 = [p.parse(open(fn, "r").read()) for fn in ["p1.ts","p2.ts"]]
    t = lambda f,ts : to_png(ts_to_dot(ts),f)
    t(open("p1.png", 'wb'), p1)
    t(open("p2.png", 'wb'), p2)

    # p1 || p2
    p1_p2 = async_product(p1,p2)
    t(open("p1_p2.png", 'wb'), p1_p2)

    print("test_ex1 completed (check p1_p2.png)")

def test_ex2():
    print("Running test_ex2...")
    p = TSParser()
    p1,p2,sv = [p.parse(open(fn, "r").read()) for fn in ["p1.ts","p2.ts","sv.ts"]]
    t = lambda f,ts : to_png(ts_to_dot(ts),f)
    t(open("sv.png", 'wb'), sv)

    # p1 || p2 || sv
    p1_p2_sv = async_product(async_product(p1, p2), sv)
    t(open("p1_p2_sv.png", 'wb'), p1_p2_sv)

    print("test_ex2 completed (check p1_p2_sv.png)")

def test_ex3():
    print("Running test_ex3...")
    p = TSParser()
    p1,p2,sv = [p.parse(open(fn, "r").read()) for fn in ["p1.ts","p2.ts","sv.ts"]]

    # p1 || p2 || sv
    p1_p2_sv = async_product(async_product(p1, p2), sv)

    # Check the following properties:
    critical1 = Symbol("critical1")
    critical2 = Symbol("critical2")

    invar_formulas = [Not(And(critical1, critical2)),
                      Not(critical1),
                      Not(critical2)]

    for invar in invar_formulas:
        res, path = check_invar(p1_p2_sv, invar)

        print("Invariant",invar.serialize(),"is",res)
        if not res and not (path is None):

            print("A counterexample showing this is:")
            print(path)

            # If this assertion fails, then paht is not a path of the transition system
            assert(p1_p2_sv.simulate(path))

    print("test_ex3 completed.")

def main():
    # P1 || P2
    test_ex1()

    # P1 || P2 || SV
    if (not os.path.exists("sv.ts")):
        print("Skipping test_ex2 and test_ex3: you have to write the file sv.ts first.")
    else:
        test_ex2()
        test_ex3()


if __name__ == "__main__":
    main()
